<?php

namespace CreoCode\ContaoCustomMailTransponder;

use Contao\DataContainer;
use Contao\Database;
use Contao\CoreBundle\ServiceAnnotation\Hook;
use Contao\Form;

use Contao\StringUtil;
use Contao\Input;

use Symfony\Bundle\SwiftmailerBundle\SwiftmailerBundle;

/**
 * Class PrepareFormDataListener
 */
class PrepareFormDataListener extends \Frontend
{
    /**
     * @Hook("prepareFormData")
     */
    public function onPrepareFormData(array &$arrSubmitted, array $arrLabels, array $arrFields, Form $form)
    {
        // $submittedData = $arrSubmitted;
        // $labels = $arrLabels
        // $fields = $arrFields
        // $form = $this

        if($form->custom_mail_transponder && $form->use_php_mail_transponder || $form->custom_mail_transponder && $form->use_swiftmailer_mail_transponder) {

            /*
                Create new emails and use the according transponder
                Almost the same function as contao/core-bundle/src/Resources/contao/forms/Form.php
            */
            $keys = array();
            $values = array();
            $fields = array();
            $message = '';

            foreach ($arrSubmitted as $k=>$v)
            {
                if ($k == 'cc')
                {
                    continue;
                }

                $v = StringUtil::deserialize($v);

                // Skip empty fields
                if ($form->skipEmpty && !\is_array($v) && !\strlen($v))
                {
                    continue;
                }

                // Add field to message
                $message .= ($arrLabels[$k] ?? ucfirst($k)) . ': ' . (\is_array($v) ? implode(', ', $v) : $v) . "\n";

                // Prepare XML file
                if ($form->format == 'xml')
                {
                    $fields[] = array
                    (
                        'name' => $k,
                        'values' => (\is_array($v) ? $v : array($v))
                    );
                }

                // Prepare CSV file
                if ($form->format == 'csv')
                {
                    $keys[] = $k;
                    $values[] = (\is_array($v) ? implode(',', $v) : $v);
                }
            }

            $recipients = StringUtil::splitCsv($form->recipient);

            // Format recipients
            foreach ($recipients as $k=>$v)
            {
                $recipients[$k] = str_replace(array('[', ']', '"'), array('<', '>', ''), $v);
            }

            //$email = new Email();
            $email = new \stdClass();

            // Get subject and message
            if ($form->format == 'email')
            {
                $message = $arrSubmitted['message'];
                $email->subject = $arrSubmitted['subject'];
            }

            // Set the admin e-mail as "from" address
            $email->from = $GLOBALS['TL_ADMIN_EMAIL'];
            $email->fromName = $GLOBALS['TL_ADMIN_NAME'];

            // Get the "reply to" address
            if (!empty(Input::post('email', true)))
            {
                $replyTo = Input::post('email', true);
                $replyToArray[Input::post('email', true)] = '';

                // Add the name
                if (!empty(Input::post('name')))
                {
                    $replyTo = '"' . Input::post('name') . '" <' . $replyTo . '>';
                    $replyToArray[Input::post('email', true)] = Input::post('name');
                }
                elseif (!empty(Input::post('firstname')) && !empty(Input::post('lastname')))
                {
                    $replyTo = '"' . Input::post('firstname') . ' ' . Input::post('lastname') . '" <' . $replyTo . '>';
                    $replyToArray[Input::post('email', true)] = Input::post('firstname') . ' ' . Input::post('lastname');
                }

                //$email->replyTo($replyTo);
            }

            // Fallback to default subject
            if (!$email->subject)
            {
                $email->subject = $form->replaceInsertTags($form->subject, false);
            }

            // Send copy to sender
            if (!empty($arrSubmitted['cc']))
            {
                //$email->sendCc(Input::post('email', true));
                $email->sendCc = Input::post('email', true);
                unset($_SESSION['FORM_DATA']['cc']);
            }

            // Attach XML file
            if ($form->format == 'xml')
            {
                $objTemplate = new FrontendTemplate('form_xml');
                $objTemplate->fields = $fields;
                $objTemplate->charset = Config::get('characterSet');

                $this->log('Attachments are currently not supported in the "Custom Mail Transponder Modul"', __METHOD__, TL_ERROR );
                //$email->attachFileFromString($objTemplate->parse(), 'form.xml', 'application/xml');
            }

            // Attach CSV file
            if ($form->format == 'csv')
            {
                $this->log('Attachments are currently not supported in the "Custom Mail Transponder Modul"', __METHOD__, TL_ERROR );
                //$email->attachFileFromString(StringUtil::decodeEntities('"' . implode('";"', $keys) . '"' . "\n" . '"' . implode('";"', $values) . '"'), 'form.csv', 'text/comma-separated-values');
            }

            $uploaded = '';

            // Attach uploaded files
            if (!empty($_SESSION['FILES']))
            {
                foreach ($_SESSION['FILES'] as $file)
                {
                    // Add a link to the uploaded file
                    if ($file['uploaded'])
                    {
                        $uploaded .= "\n" . Environment::get('base') . StringUtil::stripRootDir(\dirname($file['tmp_name'])) . '/' . rawurlencode($file['name']);
                        continue;
                    }

                    $this->log('Attachments are currently not supported in the "Custom Mail Transponder Modul"', __METHOD__, TL_ERROR );
                    //$email->attachFileFromString(file_get_contents($file['tmp_name']), $file['name'], $file['type']);
                }
            }

            $uploaded = \strlen(trim($uploaded)) ? "\n\n---\n" . $uploaded : '';
            $email->text = StringUtil::decodeEntities(trim($message)) . $uploaded . "\n\n";

            /*
                Send the e-mail with PHP mail
            */
            if ($form->use_php_mail_transponder) {
                $mailRecipients = implode(',', $recipients);
                $mailSubject = $email->subject;
                $mailContent = $email->text;
                $mailHeaders = 'From: ' . $email->fromName . ' <' . $email->from . '>' . "\r\n";
                $mailHeaders .= 'Reply-To: ' . $replyTo . "\r\n";
                if ($email->sendCc) {
                    $mailHeaders .= 'Cc: ' . $email->sendCc . "\r\n";
                }
                //$mailHeaders .= 'Content-Type: text/html' . "\r\n";
                 
                if (mail($mailRecipients, $mailSubject, $mailContent, $mailHeaders)) {
                    $strMessage = 'An e-mail (with custom mail transponder / phpmail) has been sent to ' . implode(',', $recipients);
                    if ($email->sendCc) {
                        $strMessage .= ', CC to ' . $email->sendCc;
                    }
                    $this->log($strMessage, __METHOD__, TL_FORMS);
                } else {
                    $this->log('Form (with custom mail transponder) "' . $form->title . '" could not be sent: ' . $e->getMessage(), __METHOD__, TL_ERROR);
                }
            }


            /*
                Send the e-mail with swiftmailer
            */
            if ($form->use_swiftmailer_mail_transponder) {
                if ($form->use_swiftmailer_mail_transponder_command) {
                    $transport = new \Swift_SendmailTransport($form->use_swiftmailer_mail_transponder_command);
                } else {
                    $transport = new \Swift_SendmailTransport('/usr/sbin/sendmail -bs');
                }                
                $mailer = new \Swift_Mailer($transport);
                $message = new \Swift_Message();
                $message->setTo($recipients);
                $message->setSubject($email->subject);
                $message->setBody($email->text);
                $message->setFrom([$email->from => $email->fromName]);
                $message->setReplyTo($replyToArray);
                if ($email->sendCc) {
                    $message->setCc($email->sendCc);
                }

                if ($mailer->send($message)) {
                    $strMessage = 'An e-mail (with custom mail transponder / swiftmailer) has been sent to ' . implode(',', $recipients);
                    if ($email->sendCc) {
                        $strMessage .= ', CC to ' . $email->sendCc;
                    }
                    $this->log($strMessage, __METHOD__, TL_FORMS);
                } else {
                    $this->log('Form (with custom mail transponder) "' . $form->title . '" could not be sent: ' . $e->getMessage(), __METHOD__, TL_ERROR);
                }
            }

            /*
            try
            {
                $email->sendTo($recipients);
            }
            catch (\Swift_SwiftException $e)
            {
                $this->log('Form "' . $this->title . '" could not be sent: ' . $e->getMessage(), __METHOD__, TL_ERROR);
            }
            */

            // Deactivate default sendViaEmail
            $form->sendViaEmail = false;

        }

    }
}